/*
								+----------------------------------+
								|                                  |
								|     ***  VBO wrap class  ***     |
								|                                  |
								|   Copyright  -tHE SWINe- 2005   |
								|                                  |
								|       VertexBufferObject.h       |
								|                                  |
								+----------------------------------+
*/

/*
 *	passed code revision at 08-13-2006
 *
 *	integers, contanining just true or false were changed to bool
 *	added CPixelBufferObject class
 *	changed behavior of CVertexBufferObject::UnMap() so it returns true when buffer was not mapped
 *	(thereby return walue false always mean buffer data were lost)
 *	renamed CVBOUnBinder to CBufferObjectUnBinder
 *
 */

#ifndef __VERTEX_BUFFER_OBJECT_INCLUDED
#define __VERTEX_BUFFER_OBJECT_INCLUDED

/*
 *	class CVertexBufferObject
 *		- class, wrapping GL_ARB_VERTEX_BUFFER_OBJECT extension
 *		- object of this class holds a single vertex buffer object, it is created in OpenGL when
 *		  first called to Bind(), BindAs() or Alloc()
 *		- OpenGL object is destroyed when CVertexBufferObject class object is destroyed
 */
class CVertexBufferObject {
protected:
	unsigned int m_n_buffer_id;
	GLenum m_n_primary_target;
	bool m_b_mapped;
	int m_n_size;
	int m_n_usage;
	int m_n_access;

public:
	/*
	 *	CVertexBufferObject::CVertexBufferObject(GLenum n_target = GL_ARRAY_BUFFER_ARB)
	 *		- default constructor
	 *		- valid parameters are GL_ARRAY_BUFFER_ARB (vertex data) and
	 *		  GL_ELEMENT_ARRAY_BUFFER_ARB (vertex indices)
	 */
	CVertexBufferObject(GLenum n_target = GL_ARRAY_BUFFER_ARB);

	/*
	 *	CVertexBufferObject::~CVertexBufferObject()
	 *		- default destructor
	 */
	~CVertexBufferObject();

	/*
	 *	static bool CVertexBufferObject::b_Supported()
	 *		- returns true when GL_ARB_VERTEX_BUFFER_OBJECT extension is supported
	 *		  and all entry points are located
	 */
	static bool b_Supported();

	/*
	 *	bool CVertexBufferObject::Bind()
	 *		- binds the buffer with target set to target, passed to constructor
	 *		  (when called first time, it creates OpenGL object as well)
	 *		- returns true on success, false on failure
	 */
	bool Bind();

	/*
	 *	bool CVertexBufferObject::BindAs(GLenum n_target = GL_PIXEL_PACK_BUFFER_ARB)
	 *		- binds the buffer with target n_target
	 *		  (when called first time, it creates OpenGL object as well)
	 *		- returns true on success, false on failure
	 *		- valid parameters are GL_ARRAY_BUFFER_ARB (vertex data) and
	 *		  GL_ELEMENT_ARRAY_BUFFER_ARB (vertex indices)
	 *		- note GL_PIXEL_PACK_BUFFER_ARB and GL_PIXEL_UNPACK_BUFFER_ARB are possible, but
	 *		  should only be used with CPixelBufferObject (it's essentialy the same class, but it
	 *		  makes you sure you check hardware support of this extension)
	 *		- you can copy data using glReadPixels or glGetTexImage with <data> parameter,
	 *		  returned by call to p_OffsetPointer(0)
	 */
	bool BindAs(GLenum n_target = GL_PIXEL_PACK_BUFFER_ARB);

	/*
	 *	bool CVertexBufferObject::Alloc(int n_size, const void *p_data = NULL,
	 *		GLenum n_usage = GL_STATIC_DRAW_ARB)
	 *		- alloc buffer data of size n_size bytes, can be initialized to p_data in case
	 *		  non-NULL pointer is supplied
	 *		- n_usage is optimalization hint only, it is one of:
	 *		  GL_STREAM_DRAW_ARB, GL_STREAM_READ_ARB, GL_STREAM_COPY_ARB
	 *		  GL_STATIC_DRAW_ARB, GL_STATIC_READ_ARB, GL_STATIC_COPY_ARB
	 *		  GL_DYNAMIC_DRAW_ARB, GL_DYNAMIC_READ_ARB, GL_DYNAMIC_COPY_ARB
	 *		- in case buffer was allocated already, it's data are freed as soon as there are no
	 *		  OpenGL commands, sourcing data from it (it shouldn't block program execution,
	 *		  depends on graphics card vendor and driver version) and new buffer is allocated
	 *		- it always returns true, but it may generate GL_OUT_OF_MEMORY error (call glGetError())
	 */
	bool Alloc(int n_size, const void *p_data = NULL, GLenum n_usage = GL_STATIC_DRAW_ARB);

	/*
	 *	void *CVertexBufferObject::p_OffsetPointer(int n_offset)
	 *		- returns offset pointer for use with glDrawElements, etc ... (always, can never fail)
	 *		- do not use this pointer to access data, it doesn't point to client address space
	 */
	void *p_OffsetPointer(int n_offset);

	/*
	 *	void CVertexBufferObject::BufferSubData(int n_offset, int n_size, const void *p_data)
	 *		- re-specify part (or all) of buffer data from p_data
	 *		- n_offset must be greater or equal to 0, n_offset + n_size must
	 *		  be less or equal to buffer size, otherwise OpenGL error is generated (glGetError())
	 */
	void BufferSubData(int n_offset, int n_size, const void *p_data);

	/*
	 *	void CVertexBufferObject::GetBufferSubData(int n_offset, int n_size, void *p_data)
	 *		- read part (or all) of buffer data into p_data
	 *		- n_offset must be greater or equal to 0, n_offset + n_size must
	 *		  be less or equal to buffer size, otherwise OpenGL error is generated (glGetError())
	 */
	void GetBufferSubData(int n_offset, int n_size, void *p_data);

	/*
	 *	void *CVertexBufferObject::p_Map(int n_access)
	 *		- map buffer into client address space (one buffer only can be mapped at a time)
	 *		- n_access is buffer access, it can be one of:
	 *		  GL_READ_ONLY_ARB, GL_WRITE_ONLY_ARB, GL_READ_WRITE_ARB
	 *		- return client address space pointer in case of success or NULL on failure
	 */
	void *p_Map(int n_access);

	/*
	 *	void *CVertexBufferObject::p_BufferPointer()
	 *		- in case buffer object has been successfuly mapped, return client-space pointer
	 *		  otherwise return NULL
	 */
	void *p_BufferPointer(); // return pointer only in case it's been mapped

	/*
	 *	bool CVertexBufferObject::UnMap()
	 *		- unmaps buffer, returns true on success or in case buffer was not mapped
	 *		- returns false on failure (buffer contents were lost, may be caused by
	 *		  screen-mode switch while buffer was bound)
	 */
	bool UnMap();

	/*
	 *	int CVertexBufferObject::n_Size()
	 *		- returns size of buffer in bytes
	 */
	int n_Size();

	/*
	 *	int CVertexBufferObject::n_Usage()
	 *		- returns buffer ussage; one of:
	 *		  GL_STREAM_DRAW_ARB, GL_STREAM_READ_ARB, GL_STREAM_COPY_ARB
	 *		  GL_STATIC_DRAW_ARB, GL_STATIC_READ_ARB, GL_STATIC_COPY_ARB
	 *		  GL_DYNAMIC_DRAW_ARB, GL_DYNAMIC_READ_ARB, GL_DYNAMIC_COPY_ARB
	 */
	int n_Usage();

	/*
	 *	bool CVertexBufferObject::b_Mapped()
	 *		- returns true in case buffer contents were mapped into client-space memory
	 *		- it's possible to obtain pointer to buffer data by call to p_BufferPointer()
	 */
	bool b_Mapped();

	/*
	 *	int CVertexBufferObject::n_AccessFlag()
	 *		- returns access flag to buffer in case it has been mapped (one of GL_READ_ONLY_ARB,
	 *		  GL_WRITE_ONLY_ARB, GL_READ_WRITE_ARB), in case buffer is not mapped, return 0
	 */
	int n_AccessFlag();
};

/*
 *	class CPixelBufferObject
 *		- class, wrapping GL_ARB_PIXEL_BUFFER_OBJECT extension
 *		- extension adds no new api to GL_ARB_VERTEX_BUFFER_OBJECT, there are only two new
 *		  bind targets (GL_PIXEL_PACK_BUFFER_ARB, GL_PIXEL_UNPACK_BUFFER_ARB)
 *		- this class adds different default constructor parameter and checks for
 *		  GL_ARB_PIXEL_BUFFER_OBJECT extension support in b_Supported()
 */
class CPixelBufferObject : public CVertexBufferObject {
public:
	/*
	 *	CPixelBufferObject::CPixelBufferObject(GLenum n_target = GL_PIXEL_PACK_BUFFER_ARB)
	 *		- default constructor, valid parameters are GL_ARRAY_BUFFER_ARB (vertex data),
	 *		  GL_ELEMENT_ARRAY_BUFFER_ARB (vertex indices), GL_PIXEL_PACK_BUFFER_ARB
	 *		  (pixel-buffer in mode for pixel storage) and GL_PIXEL_UNPACK_BUFFER_ARB
	 *		  (pixel-buffer in mode for for reading pixels)
	 */
	CPixelBufferObject(GLenum n_target = GL_PIXEL_PACK_BUFFER_ARB);

	/*
	 *	static bool CPixelBufferObject::b_Supported()
	 *		- returns true when GL_ARB_VERTEX_BUFFER_OBJECT and GL_ARB_PIXEL_BUFFER_OBJECT
	 *		  extensions are supported and all entry points were found
	 */
	static bool b_Supported();
};

class CBufferObjectUnBinder {
public:
	static void UnBind(GLenum n_target = GL_ARRAY_BUFFER_ARB) { glBindBufferARB(n_target, 0); }
};

#endif //__VERTEX_BUFFER_OBJECT_INCLUDED
